@@ -557,15 +557,9 @@ burn_stack (int bytes)
 }
 
 
-
 /* Perform the key setup.  Note that this works only with 128- and 256-bit
  * keys, despite the API that looks like it might support other sizes. */
 
-#ifdef __riscos__
-/* need to switch off CSE optimisation for Norcroft C (Acorn/Pace) */
-#pragma no_optimise_cse
-#endif /* __riscos__ */
-
 static int
 do_twofish_setkey (TWOFISH_context *ctx, const byte *key, unsigned int keylen)
 {
@@ -701,12 +695,8 @@ do_twofish_setkey (TWOFISH_context *ctx, const byte *key, unsigned int keylen)
     return 0;
 }
 
-#ifdef __riscos__
-#pragma optimise_cse
-#endif /* __riscos__ */
-
 static int
-twofish_setkey (TWOFISH_context *ctx, const byte *key, unsigned int keylen)
+twofish_setkey (void *ctx, const byte *key, unsigned int keylen)
 {
     int rc = do_twofish_setkey (ctx, key, keylen);
     burn_stack (23+6*sizeof(void*));
@@ -775,11 +765,6 @@ twofish_setkey (TWOFISH_context *ctx, const byte *key, unsigned int keylen)
 
 /* Encrypt one block.  in and out may be the same. */
 
-#ifdef __riscos__
-/* need to switch off CSE optimisation for Norcroft C (Acorn/Pace) */
-#pragma no_optimise_cse
-#endif /* __riscos__ */
-
 static void
 do_twofish_encrypt (const TWOFISH_context *ctx, byte *out, const byte *in)
 {
@@ -812,12 +797,8 @@ do_twofish_encrypt (const TWOFISH_context *ctx, byte *out, const byte *in)
    OUTUNPACK (3, b, 7);
 }
 
-#ifdef __riscos__
-#pragma optimise_cse
-#endif /* __riscos__ */
-
 static void
-twofish_encrypt (const TWOFISH_context *ctx, byte *out, const byte *in)
+twofish_encrypt (void *ctx, byte *out, const byte *in)
 {
     do_twofish_encrypt (ctx, out, in);
     burn_stack (24+3*sizeof (void*));
@@ -825,11 +806,6 @@ twofish_encrypt (const TWOFISH_context *ctx, byte *out, const byte *in)
 
 /* Decrypt one block.  in and out may be the same. */
 
-#ifdef __riscos__
-/* need to switch off CSE optimisation for Norcroft C (Acorn/Pace) - bug */
-#pragma no_optimise_cse
-#endif /* __riscos__ */
-
 static void
 do_twofish_decrypt (const TWOFISH_context *ctx, byte *out, const byte *in)
 {
@@ -862,12 +838,8 @@ do_twofish_decrypt (const TWOFISH_context *ctx, byte *out, const byte *in)
    OUTUNPACK (3, d, 3);
 }
 
-#ifdef __riscos__
-#pragma optimise_cse
-#endif /* __riscos__ */
-
 static void
-twofish_decrypt (const TWOFISH_context *ctx, byte *out, const byte *in)
+twofish_decrypt (void *ctx, byte *out, const byte *in)
 {
     do_twofish_decrypt (ctx, out, in);
     burn_stack (24+3*sizeof (void*));
@@ -1036,28 +1008,22 @@ main()
 }
 
 #endif /* TEST */
-
-#ifdef IS_MODULE
-static
-#endif
-       const char *
-twofish_get_info (int algo, size_t *keylen,
-		  size_t *blocksize, size_t *contextsize,
-		  int  (**r_setkey) (void *c, byte *key, unsigned keylen),
-		  void (**r_encrypt) (void *c, byte *outbuf, byte *inbuf),
-		  void (**r_decrypt) (void *c, byte *outbuf, byte *inbuf)
+
+const char *
+twofish_get_info(int algo, size_t *keylen,
+		 size_t *blocksize, size_t *contextsize,
+		 int (**r_setkey) (void *c, const byte *key, unsigned keylen),
+		 void (**r_encrypt) (void *c, byte *outbuf, const byte *inbuf),
+		 void (**r_decrypt) (void *c, byte *outbuf, const byte *inbuf)
 		 )
 {
     *keylen = algo==10? 256 : 128;
     *blocksize = 16;
     *contextsize = sizeof (TWOFISH_context);
 
-    *(int  (**)(TWOFISH_context*, const byte*, const unsigned))r_setkey
-							= twofish_setkey;
-    *(void (**)(const TWOFISH_context*, byte*, const byte*))r_encrypt
-							= twofish_encrypt;
-    *(void (**)(const TWOFISH_context*, byte*, const byte*))r_decrypt
-							= twofish_decrypt;
+    *r_setkey = twofish_setkey;
+    *r_encrypt = twofish_encrypt;
+    *r_decrypt = twofish_decrypt;
 
     if( algo == 10 )
 	return "TWOFISH";
@@ -1066,70 +1032,3 @@ twofish_get_info (int algo, size_t *keylen,
 	return "TWOFISH128";
     return NULL;
 }
-
-
-#ifdef IS_MODULE
-static
-const char * const gnupgext_version = "TWOFISH ($Revision: 1731 $)";
-
-static struct {
-    int class;
-    int version;
-    int  value;
-    void (*func)(void);
-} func_table[] = {
-    { 20, 1, 0, (void(*)(void))twofish_get_info },
-    { 21, 1, 10  },
-    { 21, 1, 102 },
-};
-
-
-
-/****************
- * Enumerate the names of the functions together with information about
- * this function. Set sequence to an integer with a initial value of 0 and
- * do not change it.
- * If what is 0 all kind of functions are returned.
- * Return values: class := class of function:
- *			   10 = message digest algorithm info function
- *			   11 = integer with available md algorithms
- *			   20 = cipher algorithm info function
- *			   21 = integer with available cipher algorithms
- *			   30 = public key algorithm info function
- *			   31 = integer with available pubkey algorithms
- *		  version = interface version of the function/pointer
- *			    (currently this is 1 for all functions)
- */
-static void *
-gnupgext_enum_func ( int what, int *sequence, int *class, int *vers )
-{
-    void *ret;
-    int i = *sequence;
-
-    do {
-	if ( i >= DIM(func_table) || i < 0 ) {
-	    return NULL;
-	}
-	*class = func_table[i].class;
-	*vers  = func_table[i].version;
-	switch( *class ) {
-	  case 11:
-	  case 21:
-	  case 31:
-	    ret = &func_table[i].value;
-	    break;
-	  default:
-#ifndef __riscos__
-	    ret = func_table[i].func;
-#else /* __riscos__ */
-	    ret = (void *) func_table[i].func;
-#endif /* __riscos__ */
-	    break;
-	}
-	i++;
-    } while ( what && what != *class );
-
-    *sequence = i;
-    return ret;
-}
-#endif
