@@ -50,7 +50,7 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
  * OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: ccid-driver.h 4603 2007-10-23 10:48:09Z wk $
+ * $Id: ccid-driver.h 5085 2009-07-21 14:30:13Z wk $
  */
 
 #ifndef CCID_DRIVER_H
@@ -80,6 +80,9 @@ typedef struct ccid_driver_s *ccid_driver_t;
 int ccid_set_debug_level (int level);
 char *ccid_get_reader_list (void);
 int ccid_open_reader (ccid_driver_t *handle, const char *readerid);
+int ccid_set_progress_cb (ccid_driver_t handle, 
+                          void (*cb)(void *, const char *, int, int, int),
+                          void *cb_arg);
 int ccid_shutdown_reader (ccid_driver_t handle);
 int ccid_close_reader (ccid_driver_t handle);
 int ccid_get_atr (ccid_driver_t handle,
