@@ -6,7 +6,7 @@
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -76,6 +74,10 @@ overwrite_filep( const char *fname )
     if ( !strcmp ( fname, "/dev/null" ) )
         return 1; /* does not do any harm */
 #endif
+#ifdef HAVE_W32_SYSTEM
+    if ( !strcmp ( fname, "nul" ) )
+        return 1;
+#endif
 
     /* fixme: add some backup stuff in case of overwrite */
     if( opt.answer_yes )
@@ -193,8 +195,14 @@ open_outfile( const char *iname, int mode, IOBUF *a )
     char *buf = NULL;
     const char *name;
     
-    if( opt.dry_run )
-      name = "/dev/null";
+    if ( opt.dry_run )
+      {
+#ifdef HAVE_W32_SYSTEM
+        name = "nul";
+#else
+        name = "/dev/null";
+#endif
+      }
     else if( opt.outfile )
       name = opt.outfile;
     else {
