@@ -1,11 +1,11 @@
 /* photoid.c - photo ID handling code
- * Copyright (C) 2001, 2002 Free Software Foundation, Inc.
+ * Copyright (C) 2001, 2002, 2005, 2006, 2008 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -14,9 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -41,17 +39,20 @@
 #include "main.h"
 #include "photoid.h"
 #include "ttyio.h"
+#include "trustdb.h"
 
 /* Generate a new photo id packet, or return NULL if canceled */
-PKT_user_id *generate_photo_id(PKT_public_key *pk)
+PKT_user_id *
+generate_photo_id(PKT_public_key *pk,const char *photo_name)
 {
   PKT_user_id *uid;
   int error=1,i;
   unsigned int len;
-  char *filename=NULL;
+  char *filename;
   byte *photo=NULL;
   byte header[16];
   IOBUF file;
+  int overflow;
 
   header[0]=0x10; /* little side of photo header length */
   header[1]=0;    /* big side of photo header length */
@@ -62,26 +63,45 @@ PKT_user_id *generate_photo_id(PKT_public_key *pk)
     header[i]=0;
 
 #define EXTRA_UID_NAME_SPACE 71
-  uid=m_alloc_clear(sizeof(*uid)+71);
+  uid=xmalloc_clear(sizeof(*uid)+71);
 
-  tty_printf(_("\nPick an image to use for your photo ID.  "
-           "The image must be a JPEG file.\n"
-	   "Remember that the image is stored within your public key.  "
-	   "If you use a\n"
-	   "very large picture, your key will become very large as well!\n"
-	   "Keeping the image close to 240x288 is a good size to use.\n"));
+  if(photo_name && *photo_name)
+    filename=make_filename(photo_name,(void *)NULL);
+  else
+    {
+      tty_printf(_("\nPick an image to use for your photo ID."
+		   "  The image must be a JPEG file.\n"
+		   "Remember that the image is stored within your public key."
+		   "  If you use a\n"
+		   "very large picture, your key will become very large"
+		   " as well!\n"
+		   "Keeping the image close to 240x288 is a good size"
+		   " to use.\n"));
+      filename=NULL;
+    }
 
   while(photo==NULL)
     {
-      tty_printf("\n");
+      if(filename==NULL)
+	{
+	  char *tempname;
 
-      m_free(filename);
+	  tty_printf("\n");
 
-      filename=cpr_get("photoid.jpeg.add",
-		       _("Enter JPEG filename for photo ID: "));
+	  tty_enable_completion(NULL);
 
-      if(strlen(filename)==0)
-	goto scram;
+	  tempname=cpr_get("photoid.jpeg.add",
+			   _("Enter JPEG filename for photo ID: "));
+
+	  tty_disable_completion();
+
+	  filename=make_filename(tempname,(void *)NULL);
+
+	  xfree(tempname);
+
+	  if(strlen(filename)==0)
+	    goto scram;
+	}
 
       file=iobuf_open(filename);
       if (file && is_secured_file (iobuf_get_fd (file)))
@@ -94,22 +114,27 @@ PKT_user_id *generate_photo_id(PKT_public_key *pk)
 	{
 	  log_error(_("unable to open JPEG file `%s': %s\n"),
 		    filename,strerror(errno));
+	  xfree(filename);
+	  filename=NULL;
 	  continue;
 	}
 
-      len=iobuf_get_filelength(file);
-      if(len>6144)
+      
+      len=iobuf_get_filelength(file, &overflow);
+      if(len>6144 || overflow)
 	{
 	  tty_printf( _("This JPEG is really large (%d bytes) !\n"),len);
 	  if(!cpr_get_answer_is_yes("photoid.jpeg.size",
 			    _("Are you sure you want to use it? (y/N) ")))
 	  {
 	    iobuf_close(file);
+	    xfree(filename);
+	    filename=NULL;
 	    continue;
 	  }
 	}
 
-      photo=m_alloc(len);
+      photo=xmalloc(len);
       iobuf_read(file,photo,len);
       iobuf_close(file);
 
@@ -118,8 +143,10 @@ PKT_user_id *generate_photo_id(PKT_public_key *pk)
 	 photo[6]!='J' || photo[7]!='F' || photo[8]!='I' || photo[9]!='F')
 	{
 	  log_error(_("`%s' is not a JPEG file\n"),filename);
-	  m_free(photo);
+	  xfree(photo);
 	  photo=NULL;
+	  xfree(filename);
+	  filename=NULL;
 	  continue;
 	}
 
@@ -132,7 +159,7 @@ PKT_user_id *generate_photo_id(PKT_public_key *pk)
          "user" may not be able to dismiss a viewer window! */
       if(opt.command_fd==-1)
 	{
-	  show_photos(uid->attribs,uid->numattribs,pk,NULL);
+	  show_photos(uid->attribs,uid->numattribs,pk,NULL,uid);
 	  switch(cpr_get_answer_yes_no_quit("photoid.jpeg.okay",
 					 _("Is this photo correct (y/N/q)? ")))
 	    {
@@ -140,8 +167,10 @@ PKT_user_id *generate_photo_id(PKT_public_key *pk)
 	      goto scram;
 	    case 0:
 	      free_attributes(uid);
-	      m_free(photo);
+	      xfree(photo);
 	      photo=NULL;
+	      xfree(filename);
+	      filename=NULL;
 	      continue;
 	    }
 	}
@@ -151,13 +180,13 @@ PKT_user_id *generate_photo_id(PKT_public_key *pk)
   uid->ref=1;
 
  scram:
-  m_free(filename);
-  m_free(photo);
+  xfree(filename);
+  xfree(photo);
 
   if(error)
     {
       free_attributes(uid);
-      m_free(uid);
+      xfree(uid);
       return NULL;
     }
 
@@ -229,7 +258,8 @@ char *image_type_to_string(byte type,int style)
 }
 
 #if !defined(FIXED_PHOTO_VIEWER) && !defined(DISABLE_PHOTO_VIEWER)
-static const char *get_default_photo_command(void)
+static const char *
+get_default_photo_command(void)
 {
 #if defined(_WIN32)
   OSVERSIONINFO osvi;
@@ -248,13 +278,22 @@ static const char *get_default_photo_command(void)
 #elif defined(__riscos__)
   return "Filer_Run %I";
 #else
-  return "xloadimage -fork -quiet -title 'KeyID 0x%k' stdin";
+  if(path_access("xloadimage",X_OK)==0)
+    return "xloadimage -fork -quiet -title 'KeyID 0x%k' stdin";
+  else if(path_access("eog",X_OK)==0)
+    return "eog %i";
+  else if(path_access("display",X_OK)==0)
+    return "display -title 'KeyID 0x%k' %i";
+  else
+    return "";
 #endif
 }
 #endif
 
-void show_photos(const struct user_attribute *attrs,
-		 int count,PKT_public_key *pk,PKT_secret_key *sk)
+void
+show_photos(const struct user_attribute *attrs,
+	    int count,PKT_public_key *pk,PKT_secret_key *sk,
+	    PKT_user_id *uid)
 {
 #ifndef DISABLE_PHOTO_VIEWER
   int i;
@@ -265,6 +304,8 @@ void show_photos(const struct user_attribute *attrs,
   memset(&args,0,sizeof(args));
   args.pk=pk;
   args.sk=sk;
+  args.validity_info=get_validity_info(pk,uid);
+  args.validity_string=get_validity_string(pk,uid);
 
   if(pk)
     keyid_from_pk(pk,kid);
@@ -286,12 +327,18 @@ void show_photos(const struct user_attribute *attrs,
 	  opt.photo_viewer=get_default_photo_command();
 #endif
 
+	if(!*opt.photo_viewer)
+	  {
+	    log_info(_("no photo viewer set\n"));
+	    goto fail;
+	  }
+
 	/* make command grow */
 	command=pct_expando(opt.photo_viewer,&args);
 	if(!command)
 	  goto fail;
 
-	name=m_alloc(16+strlen(EXTSEP_S)+
+	name=xmalloc(16+strlen(EXTSEP_S)+
 		     strlen(image_type_to_string(args.imagetype,0))+1);
 
 	/* Make the filename.  Notice we are not using the image
@@ -310,7 +357,7 @@ void show_photos(const struct user_attribute *attrs,
 
 	if(exec_write(&spawn,NULL,command,name,1,1)!=0)
 	  {
-	    m_free(name);
+	    xfree(name);
 	    goto fail;
 	  }
 
@@ -319,7 +366,7 @@ void show_photos(const struct user_attribute *attrs,
                                         image_type_to_string(args.imagetype,2));
 #endif
 
-	m_free(name);
+	xfree(name);
 
 	fwrite(&attrs[i].data[offset],attrs[i].len-offset,1,spawn->tochild);
 
