@@ -1,11 +1,11 @@
 /* seckey-cert.c -  secret key certificate packet handling
- * Copyright (C) 1998,1999,2000,2001,2002,2003 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -14,8 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -36,7 +35,8 @@
 
 
 static int
-do_check( PKT_secret_key *sk, const char *tryagain_text, int *canceled )
+do_check( PKT_secret_key *sk, const char *tryagain_text, int mode,
+          int *canceled )
 {
     byte *buffer;
     u16 csum=0;
@@ -49,7 +49,7 @@ do_check( PKT_secret_key *sk, const char *tryagain_text, int *canceled )
 	CIPHER_HANDLE cipher_hd=NULL;
 	PKT_secret_key *save_sk;
 
-	if( sk->protect.s2k.mode == 1001 || sk->protect.s2k.mode == 1002 ) {
+	if( sk->protect.s2k.mode == 1001 ) {
 	    log_info(_("secret key parts are not available\n"));
 	    return G10ERR_GENERAL;
 	}
@@ -78,20 +78,20 @@ do_check( PKT_secret_key *sk, const char *tryagain_text, int *canceled )
             keyid[3] = sk->main_keyid[1];
 	}
 	dek = passphrase_to_dek( keyid, sk->pubkey_algo, sk->protect.algo,
-				 &sk->protect.s2k, 0,
-                                 tryagain_text, canceled);
+				 &sk->protect.s2k, mode,
+                                 tryagain_text, canceled );
         if (!dek && canceled && *canceled)
 	    return G10ERR_GENERAL;
 
 	cipher_hd = cipher_open( sk->protect.algo,
 				 CIPHER_MODE_AUTO_CFB, 1);
 	cipher_setkey( cipher_hd, dek->key, dek->keylen );
-	m_free(dek);
+	xfree(dek);
 	save_sk = copy_secret_key( NULL, sk );
 	cipher_setiv( cipher_hd, sk->protect.iv, sk->protect.ivlen );
 	csum = 0;
 	if( sk->version >= 4 ) {
-	    int ndata;
+	    unsigned int ndata;
 	    byte *p, *data;
             u16 csumc = 0;
 
@@ -100,7 +100,7 @@ do_check( PKT_secret_key *sk, const char *tryagain_text, int *canceled )
 	    p = mpi_get_opaque( sk->skey[i], &ndata );
             if ( ndata > 1 )
                 csumc = p[ndata-2] << 8 | p[ndata-1];
-	    data = m_alloc_secure( ndata );
+	    data = xmalloc_secure( ndata );
 	    cipher_decrypt( cipher_hd, data, p, ndata );
 	    mpi_free( sk->skey[i] ); sk->skey[i] = NULL ;
 	    p = data;
@@ -145,43 +145,56 @@ do_check( PKT_secret_key *sk, const char *tryagain_text, int *canceled )
                     }
                 }
             }
-                
-            /* must check it here otherwise the mpi_read_xx would fail
+
+            /* Must check it here otherwise the mpi_read_xx would fail
                because the length may have an arbitrary value */
             if( sk->csum == csum ) {
                 for( ; i < pubkey_get_nskey(sk->pubkey_algo); i++ ) {
                     nbytes = ndata;
                     sk->skey[i] = mpi_read_from_buffer(p, &nbytes, 1 );
+                    if (!sk->skey[i])
+                      {
+                        /* Checksum was okay, but not correctly
+                           decrypted.  */
+                        sk->csum = 0;
+                        csum = 1;
+                        break;
+                      }
                     ndata -= nbytes;
                     p += nbytes;
                 }
                 /* Note: at this point ndata should be 2 for a simple
                    checksum or 20 for the sha1 digest */
             }
-	    m_free(data);
+	    xfree(data);
 	}
 	else {
 	    for(i=pubkey_get_npkey(sk->pubkey_algo);
 		    i < pubkey_get_nskey(sk->pubkey_algo); i++ ) {
                 byte *p;
-                int ndata;
-                unsigned int dummy;
+                unsigned int ndata;
 
                 assert (mpi_is_opaque (sk->skey[i]));
                 p = mpi_get_opaque (sk->skey[i], &ndata);
                 assert (ndata >= 2);
                 assert (ndata == ((p[0] << 8 | p[1]) + 7)/8 + 2);
-                buffer = m_alloc_secure (ndata);
+                buffer = xmalloc_secure (ndata);
 		cipher_sync (cipher_hd);
                 buffer[0] = p[0];
                 buffer[1] = p[1];
                 cipher_decrypt (cipher_hd, buffer+2, p+2, ndata-2);
                 csum += checksum (buffer, ndata);
                 mpi_free (sk->skey[i]);
-                dummy = ndata;
-                sk->skey[i] = mpi_read_from_buffer (buffer, &dummy, 1);
-                assert (sk->skey[i]);
-		m_free (buffer);
+                sk->skey[i] = mpi_read_from_buffer (buffer, &ndata, 1);
+		xfree (buffer);
+                if (!sk->skey[i])
+                  {
+                    /* Checksum was okay, but not correctly
+                       decrypted.  */
+                    sk->csum = 0;
+                    csum = 1;
+                    break;
+                  }
 /*  		csum += checksum_mpi (sk->skey[i]); */
 	    }
 	}
@@ -189,7 +202,7 @@ do_check( PKT_secret_key *sk, const char *tryagain_text, int *canceled )
 	/* now let's see whether we have used the right passphrase */
 	if( csum != sk->csum ) {
 	    copy_secret_key( sk, save_sk );
-            passphrase_clear_cache ( keyid, sk->pubkey_algo );
+            passphrase_clear_cache ( keyid, NULL, sk->pubkey_algo );
 	    free_secret_key( save_sk );
 	    return G10ERR_BAD_PASS;
 	}
@@ -197,7 +210,7 @@ do_check( PKT_secret_key *sk, const char *tryagain_text, int *canceled )
 	res = pubkey_check_secret_key( sk->pubkey_algo, sk->skey );
 	if( res ) {
 	    copy_secret_key( sk, save_sk );
-            passphrase_clear_cache ( keyid, sk->pubkey_algo );
+            passphrase_clear_cache ( keyid, NULL, sk->pubkey_algo );
 	    free_secret_key( save_sk );
 	    return G10ERR_BAD_PASS;
 	}
@@ -222,24 +235,36 @@ do_check( PKT_secret_key *sk, const char *tryagain_text, int *canceled )
 /****************
  * Check the secret key
  * Ask up to 3 (or n) times for a correct passphrase
+ * If n is negative, disable the key info prompt and make n=abs(n)
  */
 int
 check_secret_key( PKT_secret_key *sk, int n )
 {
     int rc = G10ERR_BAD_PASS;
-    int i;
+    int i,mode;
+
+    if (sk && sk->is_protected && sk->protect.s2k.mode == 1002)
+      return 0; /* Let the card support stuff handle this. */
+
+    if(n<0)
+      {
+	n=abs(n);
+	mode=1;
+      }
+    else
+      mode=0;
 
     if( n < 1 )
 	n = (opt.batch && !opt.use_agent)? 1 : 3; /* use the default value */
 
     for(i=0; i < n && rc == G10ERR_BAD_PASS; i++ ) {
-        int canceled;
+        int canceled = 0;
         const char *tryagain = NULL;
 	if (i) {
             tryagain = N_("Invalid passphrase; please try again");
             log_info (_("%s ...\n"), _(tryagain));
         }
-	rc = do_check( sk, tryagain, &canceled );
+	rc = do_check( sk, tryagain, mode, &canceled );
 	if( rc == G10ERR_BAD_PASS && is_status_enabled() ) {
 	    u32 kid[2];
 	    char buf[50];
@@ -248,7 +273,7 @@ check_secret_key( PKT_secret_key *sk, int n )
 	    sprintf(buf, "%08lX%08lX", (ulong)kid[0], (ulong)kid[1]);
 	    write_status_text( STATUS_BAD_PASSPHRASE, buf );
 	}
-	if( have_static_passphrase() || canceled )
+	if( have_static_passphrase() || canceled)
 	    break;
     }
 
@@ -261,11 +286,15 @@ check_secret_key( PKT_secret_key *sk, int n )
 /****************
  * check whether the secret key is protected.
  * Returns: 0 not protected, -1 on error or the protection algorithm
+ *                           -2 indicates a card stub.
+ *                           -3 indicates a not-online stub.
  */
 int
 is_secret_key_protected( PKT_secret_key *sk )
 {
-    return sk->is_protected? sk->protect.algo : 0;
+    return sk->is_protected?
+               sk->protect.s2k.mode == 1002? -2 :
+               sk->protect.s2k.mode == 1001? -3 : sk->protect.algo : 0;
 }
 
 
@@ -320,7 +349,7 @@ protect_secret_key( PKT_secret_key *sk, DEK *dek )
 		    bufarr[j] = NULL;
 		ndata += opt.simple_sk_checksum? 2 : 20; /* for checksum */
 
-		data = m_alloc_secure( ndata );
+		data = xmalloc_secure( ndata );
 		p = data;
 		for(j=0; j < PUBKEY_MAX_NSKEY && bufarr[j]; j++ ) {
 		    p[0] = nbits[j] >> 8 ;
@@ -328,7 +357,7 @@ protect_secret_key( PKT_secret_key *sk, DEK *dek )
 		    p += 2;
 		    memcpy(p, bufarr[j], narr[j] );
 		    p += narr[j];
-		    m_free(bufarr[j]);
+		    xfree(bufarr[j]);
 		}
                 
                 if (opt.simple_sk_checksum) {
@@ -374,13 +403,13 @@ protect_secret_key( PKT_secret_key *sk, DEK *dek )
 		    buffer = mpi_get_buffer( sk->skey[i], &nbytes, NULL );
 		    cipher_sync (cipher_hd);
 		    assert ( !mpi_is_opaque (sk->skey[i]) );
-                    data = m_alloc (nbytes+2);
+                    data = xmalloc (nbytes+2);
                     nbits  = mpi_get_nbits (sk->skey[i]);
                     assert (nbytes == (nbits + 7)/8);
                     data[0] = nbits >> 8;
                     data[1] = nbits;
 		    cipher_encrypt (cipher_hd, data+2, buffer, nbytes);
-		    m_free( buffer );
+		    xfree( buffer );
                     
                     mpi_free (sk->skey[i]);
                     sk->skey[i] = mpi_set_opaque (NULL, data, nbytes+2);
