@@ -1,12 +1,12 @@
 /* sig-check.c -  Check a signature
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003,
- *               2004 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
+ *               2007 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -126,19 +124,31 @@ signature_check2( PKT_signature *sig, MD_HANDLE digest, u32 *r_expiredate,
 	 * and the timestamp, but the drawback of this is, that it is
 	 * not possible to sign more than one identical document within
 	 * one second.	Some remote batch processing applications might
-	 * like this feature here */
+	 * like this feature here.
+         *
+         * Note that before 1.4.10, we used RIPE-MD160 for the hash
+         * and accidently didn't include the timestamp and algorithm
+         * information in the hash.  Given that this feature is not
+         * commonly used and that a replay attacks detection should
+         * not solely be based on this feature (because it does not
+         * work with RSA), we take the freedom and switch to SHA-1
+         * with 1.4.10 to take advantage of hardware supported SHA-1
+         * implementations and to match the 2.0.10 behaviour.  We also
+         * include the missing information in the hash.  Note also the
+         * SIG_ID as computed by gpg 1.x and gpg 2.x didn't matched
+         * either because 2.x used to print MPIs not in PGP format.  */
 	MD_HANDLE md;
 	u32 a = sig->timestamp;
 	int i, nsig = pubkey_get_nsig( sig->pubkey_algo );
 	byte *p, *buffer;
 
-	md = md_open( DIGEST_ALGO_RMD160, 0);
-	md_putc( digest, sig->pubkey_algo );
-	md_putc( digest, sig->digest_algo );
-	md_putc( digest, (a >> 24) & 0xff );
-	md_putc( digest, (a >> 16) & 0xff );
-	md_putc( digest, (a >>	8) & 0xff );
-	md_putc( digest,  a	   & 0xff );
+	md = md_open (DIGEST_ALGO_SHA1, 0);
+	md_putc (md, sig->pubkey_algo);
+	md_putc (md, sig->digest_algo);
+	md_putc (md, (a >> 24) & 0xff);
+	md_putc (md, (a >> 16) & 0xff);
+	md_putc (md, (a >>	8) & 0xff);
+	md_putc (md,  a	& 0xff);
 	for(i=0; i < nsig; i++ ) {
 	    unsigned n = mpi_get_nbits( sig->data[i]);
 
@@ -282,6 +292,38 @@ do_check( PKT_public_key *pk, PKT_signature *sig, MD_HANDLE digest,
     rc = pubkey_verify( pk->pubkey_algo, result, sig->data, pk->pkey );
     mpi_free( result );
 
+    if(rc==G10ERR_BAD_SIGN && is_RSA(pk->pubkey_algo)
+       && sig->digest_algo==DIGEST_ALGO_SHA224)
+      {
+	/* This code is to work around a SHA-224 problem.  RFC-4880
+	   and the drafts leading up to it were published with the
+	   wrong DER prefix for SHA-224.  Unfortunately, GPG pre-1.4.8
+	   used this wrong prefix.  What this code does is take all
+	   bad RSA signatures that use SHA-224, and re-checks them
+	   using the old, incorrect, DER prefix.  Someday we should
+	   remove this code, and when we do remove it, pkcs1_encode_md
+	   can be made into a static function again.  Note that GPG2
+	   does not have this issue as it uses libgcrypt, which is
+	   being fixed while it is still a development version. */
+
+	/* The incorrect SHA-224 DER prefix used in pre-1.4.8 */
+	static byte asn[]={0x30, 0x31, 0x30, 0x0d, 0x06, 0x09, 0x60, 0x86,
+			   0x48, 0x01, 0x65, 0x03, 0x04, 0x02, 0x04, 0x05,
+			   0x00, 0x04, 0x20};
+
+	result=pkcs1_encode_md(digest,DIGEST_ALGO_SHA224,28,
+			       mpi_get_nbits(pk->pkey[0]),asn,DIM(asn));
+
+	rc=pubkey_verify(pk->pubkey_algo,result,sig->data,pk->pkey);
+	mpi_free(result);
+      }
+
+    /* Print the MD5 warning if not yet done.  Thus at most we get one
+       warning during signature checking.  Note that while validating
+       a key we might have already checked MD5 key signatures.  */
+    if (sig->digest_algo == DIGEST_ALGO_MD5)
+      md5_digest_warn (0);  
+
     if( !rc && sig->flags.unknown_critical )
       {
 	log_info(_("assuming bad signature from key %s"
@@ -403,6 +445,7 @@ check_revocation_keys(PKT_public_key *pk,PKT_signature *sig)
               hash_public_key(md,pk);
               rc=signature_check(sig,md);
 	      cache_sig_result(sig,rc);
+              md_close (md);
 	      break;
 	    }
 	}
