@@ -6,7 +6,7 @@
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -470,10 +468,10 @@ init_shm_coprocessing ( ulong requested_shm_size, int lock_mem )
 
 /****************
  * Request a string from client
- * If bool, returns static string on true (do not free) or NULL for false
+ * If GETBOOL, returns static string on true (do not free) or NULL for false
  */
 static char *
-do_shm_get( const char *keyword, int hidden, int bool )
+do_shm_get( const char *keyword, int hidden, int getbool )
 {
     size_t n;
     byte *p;
@@ -487,7 +485,7 @@ do_shm_get( const char *keyword, int hidden, int bool )
     shm_area[2] = 1;  /* indicate that we are waiting on a reply */
     shm_area[3] = 0;  /* clear data available flag */
 
-    write_status_text( bool? STATUS_SHM_GET_BOOL :
+    write_status_text( getbool? STATUS_SHM_GET_BOOL :
 		       hidden? STATUS_SHM_GET_HIDDEN : STATUS_SHM_GET, keyword );
 
     do {
@@ -502,7 +500,7 @@ do_shm_get( const char *keyword, int hidden, int bool )
     if( n+32+2+1 > 4095 )
 	log_fatal("client returns too large data (%u bytes)\n", (unsigned)n );
 
-    if( bool )
+    if( getbool )
 	return p[0]? "" : NULL;
 
     string = hidden? xmalloc_secure( n+1 ) : xmalloc( n+1 );
@@ -545,10 +543,10 @@ myread(int fd, void *buf, size_t count)
 
 /****************
  * Request a string from the client over the command-fd
- * If bool, returns static string on true (do not free) or NULL for false
+ * If getbool, returns static string on true (do not free) or NULL for false
  */
 static char *
-do_get_from_fd( const char *keyword, int hidden, int bool )
+do_get_from_fd( const char *keyword, int hidden, int getbool )
 {
     int i, len;
     char *string;
@@ -556,7 +554,7 @@ do_get_from_fd( const char *keyword, int hidden, int bool )
     if(statusfp!=stdout)
       fflush(stdout);
 
-    write_status_text( bool? STATUS_GET_BOOL :
+    write_status_text( getbool? STATUS_GET_BOOL :
 		       hidden? STATUS_GET_HIDDEN : STATUS_GET_LINE, keyword );
 
     for( string = NULL, i = len = 200; ; i++ ) {
@@ -583,7 +581,7 @@ do_get_from_fd( const char *keyword, int hidden, int bool )
 
     write_status( STATUS_GOT_IT );
 
-    if( bool )	 /* Fixme: is this correct??? */
+    if( getbool )	 /* Fixme: is this correct??? */
 	return (string[0] == 'Y' || string[0] == 'y') ? "" : NULL;
 
     return string;
