@@ -592,8 +592,9 @@ open_db()
   db_fd = open (db_name, O_RDWR | MY_O_BINARY );
   if (db_fd == -1 && (errno == EACCES
 #ifdef EROFS
-                      || errno == EROFS)
+                      || errno == EROFS
 #endif
+                      )
       ) {
       db_fd = open (db_name, O_RDONLY | MY_O_BINARY );
       if (db_fd != -1)
@@ -1006,8 +1007,8 @@ drop_from_hashtable( ulong table, byte *key, int keylen, ulong recnum )
  */
 static int
 lookup_hashtable( ulong table, const byte *key, size_t keylen,
-		  int (*cmpfnc)(void*, const TRUSTREC *), void *cmpdata,
-						TRUSTREC *rec )
+		  int (*cmpfnc)(const void*, const TRUSTREC *), 
+                  const void *cmpdata, TRUSTREC *rec )
 {
     int rc;
     ulong hashrec, item;
@@ -1463,10 +1464,10 @@ tdbio_new_recnum()
 
 
 static int
-cmp_trec_fpr ( void *fpr, const TRUSTREC *rec )
+cmp_trec_fpr (const void *fpr, const TRUSTREC *rec )
 {
-    return rec->rectype == RECTYPE_TRUST
-	   && !memcmp( rec->r.trust.fingerprint, fpr, 20);
+    return (rec->rectype == RECTYPE_TRUST
+            && !memcmp( rec->r.trust.fingerprint, fpr, 20));
 }
 
 
@@ -1475,9 +1476,9 @@ tdbio_search_trust_byfpr( const byte *fingerprint, TRUSTREC *rec )
 {
     int rc;
 
-    /* locate the trust record using the hash table */
-    rc = lookup_hashtable( get_trusthashrec(), fingerprint, 20,
-			   cmp_trec_fpr, (void*)fingerprint, rec );
+    /* Locate the trust record using the hash table.  */
+    rc = lookup_hashtable (get_trusthashrec(), fingerprint, 20,
+			   cmp_trec_fpr, fingerprint, rec);
     return rc;
 }
 
@@ -1498,9 +1499,9 @@ tdbio_search_trust_bypk (PKT_public_key *pk, TRUSTREC *rec)
 void
 tdbio_invalid(void)
 {
-    log_error(_(
-	"the trustdb is corrupted; please run \"gpg --fix-trustdb\".\n") );
-    g10_exit(2);
+  log_error (_("Error: The trustdb is corrupted.\n"));
+  how_to_fix_the_trustdb ();
+  g10_exit (2);
 }
 
 /*
